# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .error import Error
from .._models import BaseModel
from .answer_response import AnswerResponse

__all__ = ["APIResponseListAnswers"]


class APIResponseListAnswers(BaseModel):
    data: Optional[List[AnswerResponse]] = None

    errors: Optional[List[Error]] = None
    """List of error messages, if any."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, Union, Mapping
from typing_extensions import Self, override

import httpx

from . import _exceptions
from ._qs import Querystring
from ._types import (
    NOT_GIVEN,
    Omit,
    Headers,
    Timeout,
    NotGiven,
    Transport,
    ProxiesTypes,
    RequestOptions,
)
from ._utils import is_given, get_async_library
from ._version import __version__
from .resources import answers, populations
from ._streaming import Stream as Stream, AsyncStream as AsyncStream
from ._exceptions import APIStatusError
from ._base_client import (
    DEFAULT_MAX_RETRIES,
    SyncAPIClient,
    AsyncAPIClient,
)

__all__ = [
    "Timeout",
    "Transport",
    "ProxiesTypes",
    "RequestOptions",
    "Semilattice",
    "AsyncSemilattice",
    "Client",
    "AsyncClient",
]


class Semilattice(SyncAPIClient):
    answers: answers.AnswersResource
    populations: populations.PopulationsResource
    with_raw_response: SemilatticeWithRawResponse
    with_streaming_response: SemilatticeWithStreamedResponse

    # client options
    api_key: str | None

    def __init__(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#client) for more details.
        http_client: httpx.Client | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new synchronous Semilattice client instance.

        This automatically infers the `api_key` argument from the `SEMILATTICE_API_KEY` environment variable if it is not provided.
        """
        if api_key is None:
            api_key = os.environ.get("SEMILATTICE_API_KEY")
        self.api_key = api_key

        if base_url is None:
            base_url = os.environ.get("SEMILATTICE_BASE_URL")
        if base_url is None:
            base_url = f"https://api.semilattice.ai"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

        self.answers = answers.AnswersResource(self)
        self.populations = populations.PopulationsResource(self)
        self.with_raw_response = SemilatticeWithRawResponse(self)
        self.with_streaming_response = SemilatticeWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        api_key = self.api_key
        if api_key is None:
            return {}
        return {"Authorization": f"Bearer {api_key}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": "false",
            **self._custom_headers,
        }

    @override
    def _validate_headers(self, headers: Headers, custom_headers: Headers) -> None:
        if self.api_key and headers.get("Authorization"):
            return
        if isinstance(custom_headers.get("Authorization"), Omit):
            return

        raise TypeError(
            '"Could not resolve authentication method. Expected the api_key to be set. Or for the `Authorization` headers to be explicitly omitted"'
        )

    def copy(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = NOT_GIVEN,
        http_client: httpx.Client | None = None,
        max_retries: int | NotGiven = NOT_GIVEN,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class AsyncSemilattice(AsyncAPIClient):
    answers: answers.AsyncAnswersResource
    populations: populations.AsyncPopulationsResource
    with_raw_response: AsyncSemilatticeWithRawResponse
    with_streaming_response: AsyncSemilatticeWithStreamedResponse

    # client options
    api_key: str | None

    def __init__(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultAsyncHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#asyncclient) for more details.
        http_client: httpx.AsyncClient | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new async AsyncSemilattice client instance.

        This automatically infers the `api_key` argument from the `SEMILATTICE_API_KEY` environment variable if it is not provided.
        """
        if api_key is None:
            api_key = os.environ.get("SEMILATTICE_API_KEY")
        self.api_key = api_key

        if base_url is None:
            base_url = os.environ.get("SEMILATTICE_BASE_URL")
        if base_url is None:
            base_url = f"https://api.semilattice.ai"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

        self.answers = answers.AsyncAnswersResource(self)
        self.populations = populations.AsyncPopulationsResource(self)
        self.with_raw_response = AsyncSemilatticeWithRawResponse(self)
        self.with_streaming_response = AsyncSemilatticeWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        api_key = self.api_key
        if api_key is None:
            return {}
        return {"Authorization": f"Bearer {api_key}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": f"async:{get_async_library()}",
            **self._custom_headers,
        }

    @override
    def _validate_headers(self, headers: Headers, custom_headers: Headers) -> None:
        if self.api_key and headers.get("Authorization"):
            return
        if isinstance(custom_headers.get("Authorization"), Omit):
            return

        raise TypeError(
            '"Could not resolve authentication method. Expected the api_key to be set. Or for the `Authorization` headers to be explicitly omitted"'
        )

    def copy(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = NOT_GIVEN,
        http_client: httpx.AsyncClient | None = None,
        max_retries: int | NotGiven = NOT_GIVEN,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class SemilatticeWithRawResponse:
    def __init__(self, client: Semilattice) -> None:
        self.answers = answers.AnswersResourceWithRawResponse(client.answers)
        self.populations = populations.PopulationsResourceWithRawResponse(client.populations)


class AsyncSemilatticeWithRawResponse:
    def __init__(self, client: AsyncSemilattice) -> None:
        self.answers = answers.AsyncAnswersResourceWithRawResponse(client.answers)
        self.populations = populations.AsyncPopulationsResourceWithRawResponse(client.populations)


class SemilatticeWithStreamedResponse:
    def __init__(self, client: Semilattice) -> None:
        self.answers = answers.AnswersResourceWithStreamingResponse(client.answers)
        self.populations = populations.PopulationsResourceWithStreamingResponse(client.populations)


class AsyncSemilatticeWithStreamedResponse:
    def __init__(self, client: AsyncSemilattice) -> None:
        self.answers = answers.AsyncAnswersResourceWithStreamingResponse(client.answers)
        self.populations = populations.AsyncPopulationsResourceWithStreamingResponse(client.populations)


Client = Semilattice

AsyncClient = AsyncSemilattice



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from semilattice import Semilattice, AsyncSemilattice
from tests.utils import assert_matches_type
from semilattice.types import APIResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestPopulations:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_create(self, client: Semilattice) -> None:
        population = client.populations.create(
            name="name",
            seed_data=b"raw file contents",
            simulation_engine="simulation_engine",
            simulation_options="simulation_options",
        )
        assert_matches_type(APIResponse, population, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_create_with_all_params(self, client: Semilattice) -> None:
        population = client.populations.create(
            name="name",
            seed_data=b"raw file contents",
            simulation_engine="simulation_engine",
            simulation_options="simulation_options",
            description="description",
            reality_target="reality_target",
            run_test=True,
        )
        assert_matches_type(APIResponse, population, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_create(self, client: Semilattice) -> None:
        response = client.populations.with_raw_response.create(
            name="name",
            seed_data=b"raw file contents",
            simulation_engine="simulation_engine",
            simulation_options="simulation_options",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        population = response.parse()
        assert_matches_type(APIResponse, population, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_create(self, client: Semilattice) -> None:
        with client.populations.with_streaming_response.create(
            name="name",
            seed_data=b"raw file contents",
            simulation_engine="simulation_engine",
            simulation_options="simulation_options",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            population = response.parse()
            assert_matches_type(APIResponse, population, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_get(self, client: Semilattice) -> None:
        population = client.populations.get(
            "population_id",
        )
        assert_matches_type(APIResponse, population, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_get(self, client: Semilattice) -> None:
        response = client.populations.with_raw_response.get(
            "population_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        population = response.parse()
        assert_matches_type(APIResponse, population, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_get(self, client: Semilattice) -> None:
        with client.populations.with_streaming_response.get(
            "population_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            population = response.parse()
            assert_matches_type(APIResponse, population, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_get(self, client: Semilattice) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `population_id` but received ''"):
            client.populations.with_raw_response.get(
                "",
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_test(self, client: Semilattice) -> None:
        population = client.populations.test(
            "population_id",
        )
        assert_matches_type(APIResponse, population, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_test(self, client: Semilattice) -> None:
        response = client.populations.with_raw_response.test(
            "population_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        population = response.parse()
        assert_matches_type(APIResponse, population, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_test(self, client: Semilattice) -> None:
        with client.populations.with_streaming_response.test(
            "population_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            population = response.parse()
            assert_matches_type(APIResponse, population, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_test(self, client: Semilattice) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `population_id` but received ''"):
            client.populations.with_raw_response.test(
                "",
            )


class TestAsyncPopulations:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip()
    @parametrize
    async def test_method_create(self, async_client: AsyncSemilattice) -> None:
        population = await async_client.populations.create(
            name="name",
            seed_data=b"raw file contents",
            simulation_engine="simulation_engine",
            simulation_options="simulation_options",
        )
        assert_matches_type(APIResponse, population, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncSemilattice) -> None:
        population = await async_client.populations.create(
            name="name",
            seed_data=b"raw file contents",
            simulation_engine="simulation_engine",
            simulation_options="simulation_options",
            description="description",
            reality_target="reality_target",
            run_test=True,
        )
        assert_matches_type(APIResponse, population, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncSemilattice) -> None:
        response = await async_client.populations.with_raw_response.create(
            name="name",
            seed_data=b"raw file contents",
            simulation_engine="simulation_engine",
            simulation_options="simulation_options",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        population = await response.parse()
        assert_matches_type(APIResponse, population, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncSemilattice) -> None:
        async with async_client.populations.with_streaming_response.create(
            name="name",
            seed_data=b"raw file contents",
            simulation_engine="simulation_engine",
            simulation_options="simulation_options",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            population = await response.parse()
            assert_matches_type(APIResponse, population, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_get(self, async_client: AsyncSemilattice) -> None:
        population = await async_client.populations.get(
            "population_id",
        )
        assert_matches_type(APIResponse, population, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_get(self, async_client: AsyncSemilattice) -> None:
        response = await async_client.populations.with_raw_response.get(
            "population_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        population = await response.parse()
        assert_matches_type(APIResponse, population, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_get(self, async_client: AsyncSemilattice) -> None:
        async with async_client.populations.with_streaming_response.get(
            "population_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            population = await response.parse()
            assert_matches_type(APIResponse, population, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_get(self, async_client: AsyncSemilattice) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `population_id` but received ''"):
            await async_client.populations.with_raw_response.get(
                "",
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_test(self, async_client: AsyncSemilattice) -> None:
        population = await async_client.populations.test(
            "population_id",
        )
        assert_matches_type(APIResponse, population, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_test(self, async_client: AsyncSemilattice) -> None:
        response = await async_client.populations.with_raw_response.test(
            "population_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        population = await response.parse()
        assert_matches_type(APIResponse, population, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_test(self, async_client: AsyncSemilattice) -> None:
        async with async_client.populations.with_streaming_response.test(
            "population_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            population = await response.parse()
            assert_matches_type(APIResponse, population, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_test(self, async_client: AsyncSemilattice) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `population_id` but received ''"):
            await async_client.populations.with_raw_response.test(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .answers import (
    AnswersResource,
    AsyncAnswersResource,
    AnswersResourceWithRawResponse,
    AsyncAnswersResourceWithRawResponse,
    AnswersResourceWithStreamingResponse,
    AsyncAnswersResourceWithStreamingResponse,
)
from .populations import (
    PopulationsResource,
    AsyncPopulationsResource,
    PopulationsResourceWithRawResponse,
    AsyncPopulationsResourceWithRawResponse,
    PopulationsResourceWithStreamingResponse,
    AsyncPopulationsResourceWithStreamingResponse,
)

__all__ = [
    "AnswersResource",
    "AsyncAnswersResource",
    "AnswersResourceWithRawResponse",
    "AsyncAnswersResourceWithRawResponse",
    "AnswersResourceWithStreamingResponse",
    "AsyncAnswersResourceWithStreamingResponse",
    "PopulationsResource",
    "AsyncPopulationsResource",
    "PopulationsResourceWithRawResponse",
    "AsyncPopulationsResourceWithRawResponse",
    "PopulationsResourceWithStreamingResponse",
    "AsyncPopulationsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from datetime import datetime

from .._models import BaseModel

__all__ = ["AnswerResponse"]


class AnswerResponse(BaseModel):
    id: str

    accuracy: Optional[float] = None

    created_at: datetime

    population: str

    population_name: str

    question: str

    root_mean_squared_error: Optional[float] = None

    status: str

    answer_options: Optional[List[object]] = None

    benchmark_id: Optional[str] = None

    ground_answer_counts: Union[object, object, None] = None

    ground_answer_percentages: Union[object, object, None] = None

    kullback_leibler_divergence: Optional[float] = None

    mean_absolute_error: Optional[float] = None

    mean_squared_error: Optional[float] = None

    normalised_kullback_leibler_divergence: Optional[float] = None

    prediction_finished_at: Optional[datetime] = None

    prediction_started_at: Optional[datetime] = None

    public: Optional[bool] = None

    question_options: Union[object, object, None] = None

    simulated_answer_percentages: Union[object, object, None] = None

    simulation_engine: Optional[str] = None

    test_finished_at: Optional[datetime] = None

    test_started_at: Optional[datetime] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Required, TypedDict

from .._types import FileTypes

__all__ = ["PopulationCreateParams"]


class PopulationCreateParams(TypedDict, total=False):
    name: Required[str]

    seed_data: Required[FileTypes]

    simulation_engine: Required[str]

    simulation_options: Required[str]

    description: Optional[str]

    reality_target: Optional[str]

    run_test: Optional[bool]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Mapping, Optional, cast

import httpx

from ..types import population_create_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven, FileTypes
from .._utils import extract_files, maybe_transform, deepcopy_minimal, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.api_response import APIResponse

__all__ = ["PopulationsResource", "AsyncPopulationsResource"]


class PopulationsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> PopulationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/semilattice-research/semilattice-sdk-python#accessing-raw-response-data-eg-headers
        """
        return PopulationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PopulationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/semilattice-research/semilattice-sdk-python#with_streaming_response
        """
        return PopulationsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        name: str,
        seed_data: FileTypes,
        simulation_engine: str,
        simulation_options: str,
        description: Optional[str] | NotGiven = NOT_GIVEN,
        reality_target: Optional[str] | NotGiven = NOT_GIVEN,
        run_test: Optional[bool] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> APIResponse:
        """
        Creates a new population model.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        body = deepcopy_minimal(
            {
                "name": name,
                "seed_data": seed_data,
                "simulation_engine": simulation_engine,
                "simulation_options": simulation_options,
                "description": description,
                "reality_target": reality_target,
                "run_test": run_test,
            }
        )
        files = extract_files(cast(Mapping[str, object], body), paths=[["seed_data"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return self._post(
            "/v1/populations",
            body=maybe_transform(body, population_create_params.PopulationCreateParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse,
        )

    def get(
        self,
        population_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> APIResponse:
        """
        Fetches a population, along with its status, accuracy metrics, and other
        metadata.

        Args:
          population_id: ID of the population to retrieve.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not population_id:
            raise ValueError(f"Expected a non-empty value for `population_id` but received {population_id!r}")
        return self._get(
            f"/v1/populations/{population_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse,
        )

    def test(
        self,
        population_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> APIResponse:
        """
        Triggers an accuracy test for a population model, if not yet tested.

        Args:
          population_id: ID of the population to test.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not population_id:
            raise ValueError(f"Expected a non-empty value for `population_id` but received {population_id!r}")
        return self._post(
            f"/v1/populations/{population_id}/test",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse,
        )


class AsyncPopulationsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncPopulationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/semilattice-research/semilattice-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPopulationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPopulationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/semilattice-research/semilattice-sdk-python#with_streaming_response
        """
        return AsyncPopulationsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        name: str,
        seed_data: FileTypes,
        simulation_engine: str,
        simulation_options: str,
        description: Optional[str] | NotGiven = NOT_GIVEN,
        reality_target: Optional[str] | NotGiven = NOT_GIVEN,
        run_test: Optional[bool] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> APIResponse:
        """
        Creates a new population model.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        body = deepcopy_minimal(
            {
                "name": name,
                "seed_data": seed_data,
                "simulation_engine": simulation_engine,
                "simulation_options": simulation_options,
                "description": description,
                "reality_target": reality_target,
                "run_test": run_test,
            }
        )
        files = extract_files(cast(Mapping[str, object], body), paths=[["seed_data"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return await self._post(
            "/v1/populations",
            body=await async_maybe_transform(body, population_create_params.PopulationCreateParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse,
        )

    async def get(
        self,
        population_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> APIResponse:
        """
        Fetches a population, along with its status, accuracy metrics, and other
        metadata.

        Args:
          population_id: ID of the population to retrieve.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not population_id:
            raise ValueError(f"Expected a non-empty value for `population_id` but received {population_id!r}")
        return await self._get(
            f"/v1/populations/{population_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse,
        )

    async def test(
        self,
        population_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> APIResponse:
        """
        Triggers an accuracy test for a population model, if not yet tested.

        Args:
          population_id: ID of the population to test.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not population_id:
            raise ValueError(f"Expected a non-empty value for `population_id` but received {population_id!r}")
        return await self._post(
            f"/v1/populations/{population_id}/test",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse,
        )


class PopulationsResourceWithRawResponse:
    def __init__(self, populations: PopulationsResource) -> None:
        self._populations = populations

        self.create = to_raw_response_wrapper(
            populations.create,
        )
        self.get = to_raw_response_wrapper(
            populations.get,
        )
        self.test = to_raw_response_wrapper(
            populations.test,
        )


class AsyncPopulationsResourceWithRawResponse:
    def __init__(self, populations: AsyncPopulationsResource) -> None:
        self._populations = populations

        self.create = async_to_raw_response_wrapper(
            populations.create,
        )
        self.get = async_to_raw_response_wrapper(
            populations.get,
        )
        self.test = async_to_raw_response_wrapper(
            populations.test,
        )


class PopulationsResourceWithStreamingResponse:
    def __init__(self, populations: PopulationsResource) -> None:
        self._populations = populations

        self.create = to_streamed_response_wrapper(
            populations.create,
        )
        self.get = to_streamed_response_wrapper(
            populations.get,
        )
        self.test = to_streamed_response_wrapper(
            populations.test,
        )


class AsyncPopulationsResourceWithStreamingResponse:
    def __init__(self, populations: AsyncPopulationsResource) -> None:
        self._populations = populations

        self.create = async_to_streamed_response_wrapper(
            populations.create,
        )
        self.get = async_to_streamed_response_wrapper(
            populations.get,
        )
        self.test = async_to_streamed_response_wrapper(
            populations.test,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .error import Error as Error
from .answer_param import AnswerParam as AnswerParam
from .api_response import APIResponse as APIResponse
from .answer_response import AnswerResponse as AnswerResponse
from .answer_get_response import AnswerGetResponse as AnswerGetResponse
from .answer_simulate_params import AnswerSimulateParams as AnswerSimulateParams
from .answer_benchmark_params import AnswerBenchmarkParams as AnswerBenchmarkParams
from .population_create_params import PopulationCreateParams as PopulationCreateParams
from .api_response_list_answers import APIResponseListAnswers as APIResponseListAnswers



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from semilattice import Semilattice, AsyncSemilattice
from tests.utils import assert_matches_type
from semilattice.types import AnswerGetResponse, APIResponseListAnswers

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestAnswers:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_benchmark(self, client: Semilattice) -> None:
        answer = client.answers.benchmark(
            answers={
                "question": "question",
                "question_options": {"question_type": "single-choice"},
            },
            population_id="population_id",
        )
        assert_matches_type(APIResponseListAnswers, answer, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_benchmark_with_all_params(self, client: Semilattice) -> None:
        answer = client.answers.benchmark(
            answers={
                "question": "question",
                "question_options": {
                    "question_type": "single-choice",
                    "limit": 0,
                    "question_number": 0,
                },
                "answer_options": ["string"],
                "ground_answer_counts": "string",
                "ground_answer_sample_size": 0,
            },
            population_id="population_id",
        )
        assert_matches_type(APIResponseListAnswers, answer, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_benchmark(self, client: Semilattice) -> None:
        response = client.answers.with_raw_response.benchmark(
            answers={
                "question": "question",
                "question_options": {"question_type": "single-choice"},
            },
            population_id="population_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        answer = response.parse()
        assert_matches_type(APIResponseListAnswers, answer, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_benchmark(self, client: Semilattice) -> None:
        with client.answers.with_streaming_response.benchmark(
            answers={
                "question": "question",
                "question_options": {"question_type": "single-choice"},
            },
            population_id="population_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            answer = response.parse()
            assert_matches_type(APIResponseListAnswers, answer, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_get(self, client: Semilattice) -> None:
        answer = client.answers.get(
            "question_id",
        )
        assert_matches_type(AnswerGetResponse, answer, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_get(self, client: Semilattice) -> None:
        response = client.answers.with_raw_response.get(
            "question_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        answer = response.parse()
        assert_matches_type(AnswerGetResponse, answer, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_get(self, client: Semilattice) -> None:
        with client.answers.with_streaming_response.get(
            "question_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            answer = response.parse()
            assert_matches_type(AnswerGetResponse, answer, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_get(self, client: Semilattice) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `question_id` but received ''"):
            client.answers.with_raw_response.get(
                "",
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_simulate(self, client: Semilattice) -> None:
        answer = client.answers.simulate(
            answers={
                "question": "question",
                "question_options": {"question_type": "single-choice"},
            },
            population_id="population_id",
        )
        assert_matches_type(APIResponseListAnswers, answer, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_simulate_with_all_params(self, client: Semilattice) -> None:
        answer = client.answers.simulate(
            answers={
                "question": "question",
                "question_options": {
                    "question_type": "single-choice",
                    "limit": 0,
                    "question_number": 0,
                },
                "answer_options": ["string"],
                "ground_answer_counts": "string",
                "ground_answer_sample_size": 0,
            },
            population_id="population_id",
        )
        assert_matches_type(APIResponseListAnswers, answer, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_simulate(self, client: Semilattice) -> None:
        response = client.answers.with_raw_response.simulate(
            answers={
                "question": "question",
                "question_options": {"question_type": "single-choice"},
            },
            population_id="population_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        answer = response.parse()
        assert_matches_type(APIResponseListAnswers, answer, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_simulate(self, client: Semilattice) -> None:
        with client.answers.with_streaming_response.simulate(
            answers={
                "question": "question",
                "question_options": {"question_type": "single-choice"},
            },
            population_id="population_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            answer = response.parse()
            assert_matches_type(APIResponseListAnswers, answer, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncAnswers:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip()
    @parametrize
    async def test_method_benchmark(self, async_client: AsyncSemilattice) -> None:
        answer = await async_client.answers.benchmark(
            answers={
                "question": "question",
                "question_options": {"question_type": "single-choice"},
            },
            population_id="population_id",
        )
        assert_matches_type(APIResponseListAnswers, answer, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_benchmark_with_all_params(self, async_client: AsyncSemilattice) -> None:
        answer = await async_client.answers.benchmark(
            answers={
                "question": "question",
                "question_options": {
                    "question_type": "single-choice",
                    "limit": 0,
                    "question_number": 0,
                },
                "answer_options": ["string"],
                "ground_answer_counts": "string",
                "ground_answer_sample_size": 0,
            },
            population_id="population_id",
        )
        assert_matches_type(APIResponseListAnswers, answer, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_benchmark(self, async_client: AsyncSemilattice) -> None:
        response = await async_client.answers.with_raw_response.benchmark(
            answers={
                "question": "question",
                "question_options": {"question_type": "single-choice"},
            },
            population_id="population_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        answer = await response.parse()
        assert_matches_type(APIResponseListAnswers, answer, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_benchmark(self, async_client: AsyncSemilattice) -> None:
        async with async_client.answers.with_streaming_response.benchmark(
            answers={
                "question": "question",
                "question_options": {"question_type": "single-choice"},
            },
            population_id="population_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            answer = await response.parse()
            assert_matches_type(APIResponseListAnswers, answer, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_get(self, async_client: AsyncSemilattice) -> None:
        answer = await async_client.answers.get(
            "question_id",
        )
        assert_matches_type(AnswerGetResponse, answer, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_get(self, async_client: AsyncSemilattice) -> None:
        response = await async_client.answers.with_raw_response.get(
            "question_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        answer = await response.parse()
        assert_matches_type(AnswerGetResponse, answer, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_get(self, async_client: AsyncSemilattice) -> None:
        async with async_client.answers.with_streaming_response.get(
            "question_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            answer = await response.parse()
            assert_matches_type(AnswerGetResponse, answer, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_get(self, async_client: AsyncSemilattice) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `question_id` but received ''"):
            await async_client.answers.with_raw_response.get(
                "",
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_simulate(self, async_client: AsyncSemilattice) -> None:
        answer = await async_client.answers.simulate(
            answers={
                "question": "question",
                "question_options": {"question_type": "single-choice"},
            },
            population_id="population_id",
        )
        assert_matches_type(APIResponseListAnswers, answer, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_simulate_with_all_params(self, async_client: AsyncSemilattice) -> None:
        answer = await async_client.answers.simulate(
            answers={
                "question": "question",
                "question_options": {
                    "question_type": "single-choice",
                    "limit": 0,
                    "question_number": 0,
                },
                "answer_options": ["string"],
                "ground_answer_counts": "string",
                "ground_answer_sample_size": 0,
            },
            population_id="population_id",
        )
        assert_matches_type(APIResponseListAnswers, answer, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_simulate(self, async_client: AsyncSemilattice) -> None:
        response = await async_client.answers.with_raw_response.simulate(
            answers={
                "question": "question",
                "question_options": {"question_type": "single-choice"},
            },
            population_id="population_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        answer = await response.parse()
        assert_matches_type(APIResponseListAnswers, answer, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_simulate(self, async_client: AsyncSemilattice) -> None:
        async with async_client.answers.with_streaming_response.simulate(
            answers={
                "question": "question",
                "question_options": {"question_type": "single-choice"},
            },
            population_id="population_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            answer = await response.parse()
            assert_matches_type(APIResponseListAnswers, answer, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union, Optional
from typing_extensions import Literal, Required, TypedDict

__all__ = ["AnswerParam", "QuestionOptions"]


class QuestionOptions(TypedDict, total=False):
    question_type: Required[Literal["single-choice", "multiple-choice", "open-ended"]]

    limit: Optional[int]

    question_number: Optional[int]


class AnswerParam(TypedDict, total=False):
    question: Required[str]

    question_options: Required[QuestionOptions]

    answer_options: Optional[List[str]]

    ground_answer_counts: Union[str, object, None]

    ground_answer_sample_size: Optional[int]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from .error import Error
from .._models import BaseModel

__all__ = ["APIResponse", "Data"]


class Data(BaseModel):
    id: str

    created_at: datetime

    name: str

    public: bool

    question_count: int

    simulacrum_count: int

    status: str

    avg_mean_absolute_error: Optional[float] = None

    avg_mean_squared_error: Optional[float] = None

    avg_normalised_kullback_leibler_divergence: Optional[float] = None

    description: Optional[str] = None

    reality_target: Optional[str] = None

    simulation_engine: Optional[str] = None

    test_finished_at: Optional[datetime] = None

    test_started_at: Optional[datetime] = None

    upload_filename: Optional[str] = None


class APIResponse(BaseModel):
    data: Optional[Data] = None
    """Population model data:"""

    errors: Optional[List[Error]] = None
    """List of error messages, if any."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import answer_simulate_params, answer_benchmark_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.answer_get_response import AnswerGetResponse
from ..types.api_response_list_answers import APIResponseListAnswers

__all__ = ["AnswersResource", "AsyncAnswersResource"]


class AnswersResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AnswersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/semilattice-research/semilattice-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AnswersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AnswersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/semilattice-research/semilattice-sdk-python#with_streaming_response
        """
        return AnswersResourceWithStreamingResponse(self)

    def benchmark(
        self,
        *,
        answers: answer_benchmark_params.Answers,
        population_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> APIResponseListAnswers:
        """
        Simulates the answer to a known question to evaluate population model accuracy.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/answers/benchmark",
            body=maybe_transform(
                {
                    "answers": answers,
                    "population_id": population_id,
                },
                answer_benchmark_params.AnswerBenchmarkParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponseListAnswers,
        )

    def get(
        self,
        question_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AnswerGetResponse:
        """
        Retrieves an answer simulation, along with its status and accuracy metrics if
        it's a benchmark simulation.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not question_id:
            raise ValueError(f"Expected a non-empty value for `question_id` but received {question_id!r}")
        return self._get(
            f"/v1/answers/{question_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AnswerGetResponse,
        )

    def simulate(
        self,
        *,
        answers: answer_simulate_params.Answers,
        population_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> APIResponseListAnswers:
        """
        Simulates the answer to a new question.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/answers",
            body=maybe_transform(
                {
                    "answers": answers,
                    "population_id": population_id,
                },
                answer_simulate_params.AnswerSimulateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponseListAnswers,
        )


class AsyncAnswersResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAnswersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/semilattice-research/semilattice-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAnswersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAnswersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/semilattice-research/semilattice-sdk-python#with_streaming_response
        """
        return AsyncAnswersResourceWithStreamingResponse(self)

    async def benchmark(
        self,
        *,
        answers: answer_benchmark_params.Answers,
        population_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> APIResponseListAnswers:
        """
        Simulates the answer to a known question to evaluate population model accuracy.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/answers/benchmark",
            body=await async_maybe_transform(
                {
                    "answers": answers,
                    "population_id": population_id,
                },
                answer_benchmark_params.AnswerBenchmarkParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponseListAnswers,
        )

    async def get(
        self,
        question_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AnswerGetResponse:
        """
        Retrieves an answer simulation, along with its status and accuracy metrics if
        it's a benchmark simulation.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not question_id:
            raise ValueError(f"Expected a non-empty value for `question_id` but received {question_id!r}")
        return await self._get(
            f"/v1/answers/{question_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AnswerGetResponse,
        )

    async def simulate(
        self,
        *,
        answers: answer_simulate_params.Answers,
        population_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> APIResponseListAnswers:
        """
        Simulates the answer to a new question.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/answers",
            body=await async_maybe_transform(
                {
                    "answers": answers,
                    "population_id": population_id,
                },
                answer_simulate_params.AnswerSimulateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponseListAnswers,
        )


class AnswersResourceWithRawResponse:
    def __init__(self, answers: AnswersResource) -> None:
        self._answers = answers

        self.benchmark = to_raw_response_wrapper(
            answers.benchmark,
        )
        self.get = to_raw_response_wrapper(
            answers.get,
        )
        self.simulate = to_raw_response_wrapper(
            answers.simulate,
        )


class AsyncAnswersResourceWithRawResponse:
    def __init__(self, answers: AsyncAnswersResource) -> None:
        self._answers = answers

        self.benchmark = async_to_raw_response_wrapper(
            answers.benchmark,
        )
        self.get = async_to_raw_response_wrapper(
            answers.get,
        )
        self.simulate = async_to_raw_response_wrapper(
            answers.simulate,
        )


class AnswersResourceWithStreamingResponse:
    def __init__(self, answers: AnswersResource) -> None:
        self._answers = answers

        self.benchmark = to_streamed_response_wrapper(
            answers.benchmark,
        )
        self.get = to_streamed_response_wrapper(
            answers.get,
        )
        self.simulate = to_streamed_response_wrapper(
            answers.simulate,
        )


class AsyncAnswersResourceWithStreamingResponse:
    def __init__(self, answers: AsyncAnswersResource) -> None:
        self._answers = answers

        self.benchmark = async_to_streamed_response_wrapper(
            answers.benchmark,
        )
        self.get = async_to_streamed_response_wrapper(
            answers.get,
        )
        self.simulate = async_to_streamed_response_wrapper(
            answers.simulate,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from typing_extensions import Required, TypeAlias, TypedDict

from .answer_param import AnswerParam

__all__ = ["AnswerBenchmarkParams", "Answers"]


class AnswerBenchmarkParams(TypedDict, total=False):
    answers: Required[Answers]

    population_id: Required[str]


Answers: TypeAlias = Union[AnswerParam, Iterable[AnswerParam]]
